module OrderedAttributeCategories exposing (..)

import Stats exposing (AttributeCategory)

type alias OrderedAttributeCategories =
    { points : List Int 
    , order : List AttributeCategory
    }
type alias OAC = OrderedAttributeCategories

new : OAC
new = 
    { points = [ 7, 5, 3 ]
    , order =
        [ Stats.Physical
        , Stats.Social
        , Stats.Mental
        ]
    }

type alias Priority =
    { value : Int
    , category : AttributeCategory
    }
indexOf : List AttributeCategory -> AttributeCategory -> Int
indexOf order category =
    order
    |> List.indexedMap Tuple.pair
    |> List.filter ( \x -> ( Tuple.second x ) == category )
    |> List.head
    |> Maybe.withDefault ( -1, Stats.Physical )
    |> Tuple.first

categoryAtPosition : List AttributeCategory -> Int -> AttributeCategory
categoryAtPosition order value =
    order
    |> List.indexedMap Tuple.pair
    |> List.filter ( \x -> Tuple.first x == value )
    |> List.head
    |> Maybe.withDefault ( -1, Stats.Physical )
    |> Tuple.second

changeOrder : OAC -> AttributeCategory -> Int -> OAC
changeOrder oac category newIndex =
    let
        order : List AttributeCategory
        order = oac.order

        currentIndex : Int
        currentIndex = indexOf order category

        unchangedIndex : Int
        unchangedIndex = 3 - ( newIndex + currentIndex )

        newOrder : List AttributeCategory
        newOrder = 
            if currentIndex == newIndex
            then order
            else
                [ ( unchangedIndex, categoryAtPosition order unchangedIndex )
                , ( currentIndex, categoryAtPosition order newIndex )
                , ( newIndex, categoryAtPosition order currentIndex )
                ]
                |> List.sortBy Tuple.first
                |> List.map Tuple.second
    in 
        { oac | order = newOrder }

categoryValue : OAC -> AttributeCategory -> Int
categoryValue oac category =
    let
        index : Int
        index = indexOf oac.order category
    in
        oac.points
        |> List.indexedMap Tuple.pair
        |> List.filter ( \x -> Tuple.first x == index )
        |> List.head
        |> Maybe.withDefault ( 0, 0 )
        |> Tuple.second